@connect /

set echo on

drop table run_stats;
create table run_stats ( runid varchar2(15), name varchar2(80), value int );

grant select any table to ops$tkyte;
create or replace view stats
as select 'STAT...' || a.name name, b.value
      from v$statname a, v$mystat b
     where a.statistic# = b.statistic#
    union all
    select 'LATCH.' || name,  gets
      from v$latch;
pause

/* 
Bind variable comparision -- what happens when you DON'T use em 
*/
drop table t;
create table t ( x int, y char(80) );
pause

declare
    l_start number;
    l_run1  number;
    l_run2  number;
begin
    insert into run_stats select 'before', stats.* from stats;

    l_start := dbms_utility.get_time;
    for i in 1 .. 5000
    loop
        execute immediate '
        insert into t (x,y) values ( :x, :y )' using i, i;
    end loop;
    l_run1 := (dbms_utility.get_time-l_start);
    dbms_output.put_line( l_run1 || ' hsecs' );
 
    insert into run_stats select 'after 1', stats.* from stats;
    l_start := dbms_utility.get_time;
    for i in 1 .. 5000
    loop
        execute immediate '
        insert into t (x,y) values 
		( ' || i || ', ''' || replace(i,'''','''''') || ''' )';
    end loop;
    l_run2 := (dbms_utility.get_time-l_start);
    dbms_output.put_line( l_run2 || ' hsecs' );
    dbms_output.put_line
    ( 'run 1 ran in ' || round(l_run1/l_run2*100,2) || '% of the time' );

    insert into run_stats select 'after 2', stats.* from stats;
end;
/
pause

select a.name, b.value-a.value run1, c.value-b.value run2,
       ( (c.value-b.value)-(b.value-a.value)) diff
  from run_stats a, run_stats b, run_stats c
 where a.name = b.name
   and b.name = c.name
   and a.runid = 'before'
   and b.runid = 'after 1'
   and c.runid = 'after 2'
   and (c.value-a.value) > 0
   and (c.value-b.value) <> (b.value-a.value)
   and abs( (c.value-b.value)-(b.value-a.value)) > 200
 order by abs( (c.value-b.value)-(b.value-a.value))
/


select run1, run2, diff, 'run1 is ' || round(run1/run2*100,2) || '% of run2 for latching'
from (
select sum(b.value-a.value) run1, 
       sum(c.value-b.value) run2,
       sum( ( (c.value-b.value)-(b.value-a.value)) ) diff
  from run_stats a, run_stats b, run_stats c
 where a.name = b.name
   and b.name = c.name
   and a.runid = 'before'
   and b.runid = 'after 1'
   and c.runid = 'after 2'
   and (c.value-a.value) > 0
   and (c.value-b.value) <> (b.value-a.value)
   and a.name like 'LATCH%'
   )

/
